// Restore References
/*Licensed Materials - Property of IBM

Restore Refs.dxl

 Copyright IBM Corporation 2008.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

//Originated in IBM from Professional Services

/*
This script converts full name references created by the preserve references tool back into
unique ID references which allow modules to be moved in the database.

version developed on    date        by              notes
1.0.0   DOORS 7.1 PC    01/26/05    Telelogic NA
1.1.0   DOORS 7.1 PC    02/14/05    Telelogic NA    always reports
1.2.0   DOORS 8.3 PC	01/14/09	IBM				modified to work with new link ID format

-------------------------------------------------
DISCLAIMER:

This programming tool has been thoroughly checked 
and tested at all stages of its production. 
IBM cannot accept any responsibility for 
any loss, disruption or damage to your data or 
your computer system that may occur while using 
this script. 

If you do not accept these conditions, do not use 
this customised script.
-------------------------------------------------
*/

pragma runLim, 0

// Constants

const string namePerm = "item"
const string idPerm = "itemFromID"
const string restoredRefHead = "// Restored\n"
const string refHead = "// References Preserved\n"
const string refMark = "//ref "
const string resMark = "//res "

// Variables

Project currProj
bool firstMsg
//Buffer errBuff
int endOffSet = 27
string IDPrefix = getDatabaseIdentifier() "-"

// Functions

//==================================================================================================
// This function adds the passed message to the message buffer.
//
// Called by: findRefs
//
// Calls: none
//
// Globals: firstMsg
//          errBuff
/*
void storeMsg(string msg) {
    if (firstMsg) {
        firstMsg = false
        errBuff = create
        errBuff = msg
    } else {
        errBuff += "\n"
        errBuff += msg
    }
} // storeMsg

//==================================================================================================
// This function builds a dialog to display the message buffer.
//
// Called by: findRefs
//
// Calls: none
//
// Globals: firstMsg
//          errBuff
//
void displayMsg() {
    if (firstMsg) {
        infoBox("No references needed to be restored")
    } else {
        DB errDB = create("Restore Notes", styleCentered)
        DBE errDBE = text(errDB, "", stringOf(errBuff), 600, 400, true)
        show(errDB)
        delete(errBuff)
    }
} // displayMsg
*/
//==================================================================================================
// This function returns the fullname of the item whose uniqueID was passed.
//
// Called by: convertRefs
//
// Calls: none
//
// Globals: currProj
//
string idFromName(string nameIn) {
    int pos
    Item modRef
    string modId
    Folder pFold
    bool noMod = true
    string projName = fullName(currProj)
    modRef = item(nameIn)
    if (null(modRef)) {
        for (pos = 1; pos < length(nameIn); pos++) {
            if (nameIn[pos] == '/') {
                if (nameIn[0:pos - 1] == projName) {
                    modRef = item(nameIn)
                } else {
                    pFold = folder(projName)
                    while (! null(getParentFolder(pFold))) {
                        pFold = getParentFolder(pFold)
                        if ("/" name(pFold) == projName) {
                            modRef = item(nameIn)
                            noMod = false
                        }
                    }
                    if (noMod) {
                        modRef = item(projName nameIn[pos:])
                    }
                }
                break
            }
        }
    }
    if (null(modRef)) {
        modId = "X"
    } else {
        modId = IDPrefix uniqueID(modRef)
    }
    return(modId)
} // idFromName

//==================================================================================================
//
//
// Called by:
//
// Calls: idFromName
//        storeMsg
//
// Globals: idPerm
//          refHead
//          currProj
//
bool convertRefs(string &dxlIn) {
    Buffer inBuff = create(length(dxlIn))
    Buffer codeLine = create(120)
    Buffer prevLine = create(120)
    Buffer newLine = create(120)
    Buffer newPrev = create(120)
    Buffer outBuff = create(length(dxlIn))
    string modId
    int pos, linePos, codeStart, prevPos, itemStart, rplcStart
    int digCount = 0
    int startPos = 0
    bool firstCode = true
    bool newCode = false
    bool inItem = false
    bool errOut = false
    inBuff = dxlIn
    outBuff = restoredRefHead
    for (pos = 0; pos < length(inBuff); pos++) {
        if (inBuff[pos] == '\n') {
            codeLine = inBuff[startPos:pos]
            if (startPos == 0) {
                if (stringOf(codeLine) != refHead) {
                    break
                }
            }
            codeStart = length(refMark)
            newLine = ""
            if (codeLine[0:(length(refMark) - 1)] == refMark) {
                for (prevPos = 0; prevPos < length(prevLine); prevPos++) {
                    if (prevLine[prevPos] == '"') {
                        if (inItem) {
                            modId = idFromName(prevLine[itemStart:prevPos - 1])
                            //print prevLine[itemStart] name(current Project) prevLine[itemStart+12:prevPos - 1] "\n"
                            if (modId == "X") {
                                //storeMsg("!!! Unable to rebuild reference for " prevLine[itemStart:prevPos - 1])
                                //print "!!! Unable to rebuild reference for " prevLine[itemStart:prevPos - 1] "\n"
                                errOut = true
                                break
                            }
                            for (rplcStart = codeStart; rplcStart < length(codeLine); rplcStart++) {
                                if (codeLine[rplcStart] == '"') {
                                    break
                                }
                            }
                            if (rplcStart == length(codeLine)) {
                                //storeMsg("!!! Reference lengths don't match: " prevLine[itemStart:prevPos - 1])
                                //print "!!! Reference lengths don't match: " prevLine[itemStart:prevPos - 1] "\n"
                                errOut = true
                            } else {
                                newLine += codeLine[codeStart:rplcStart]
                                newLine += modId
                                newLine += '"'
                                codeStart = rplcStart + endOffSet
                            }
                            inItem = false
                        } else {
                            inItem = true
                            itemStart = prevPos + 1
                        }
                    }
                    if (errOut) {
                        break
                    }
                }
                newLine += codeLine[codeStart:]
                newPrev = resMark
                newPrev += prevLine
                prevLine = stringOf(newPrev)
                codeLine = stringOf(newLine)
                newCode = true
            }
            if (firstCode) {
                firstCode = false
            } else {
                outBuff += prevLine
            }
            prevLine = codeLine
            startPos = pos + 1
        }
        if (errOut) {
            break
        }
    }
    if (! firstCode) {
        outBuff += prevLine
    }
    if (newCode && ! errOut) {
        outBuff += inBuff[startPos:]
        dxlIn = stringOf(outBuff)
    } else {
        dxlIn = stringOf(inBuff)
    }
    delete(inBuff)
    delete(codeLine)
    delete(newLine)
    delete(newPrev)
    delete(outBuff)
    return(newCode && ! errOut)
} // convertRefs

//==================================================================================================
//
//
// Called by: main
//
// Calls: storeMsg
//        convertRefs
//        displayMsg
//
// Globals: idPerm
//          currProj
//
void findRefs() {
    string viewName, colCode
    Column dxlCol
    Item itemRef
    Module currMod
    bool viewModified
    View thisView
    
    for itemRef in currProj do 
    {
        if (type(itemRef) == "Formal") 
        {
            currMod = read(fullName(itemRef), true)
            if (null(currMod) or ! canRead(currMod)) 
            {
                //storeMsg("!!! Unable to open " fullName(itemRef))
            } 
            else 
            {
                current = currMod
                for viewName in views currMod do 
                {
                    viewModified = false
                    thisView = view(viewName)
                    if (canModify(module(fullName(currMod)), thisView)) 
                    {
                        if (load(thisView)) {
                            for dxlCol in currMod do 
                            {
                                colCode = dxl(dxlCol)
                                if (! null(colCode)) 
                                {
                                    if (matches(idPerm, colCode)) 
                                    {
                                        if (convertRefs(colCode)) 
                                        {
                                            dxl(dxlCol, colCode)
                                            viewModified = true
                                        }
                                    }
                                }
                            }
                            if (viewModified) 
                            {
                                save(thisView)
								ViewDef viewInfo = get(thisView)
								useCurrent(viewInfo, false)
								useWindows(viewInfo, false)
								save(thisView, viewInfo)
                                //storeMsg("References restored in view '" viewName "' in " fullName(itemRef))
                            }
                        } 
                        /*else 
                        {
                            storeMsg("!!! Unable to load view '" viewName "' in " fullName(itemRef))
                        }*/
                    } 
                    /*else 
                    {
                        storeMsg("!!! No access to modify view '" viewName "' in " fullName(itemRef))
                    }*/
                }
                close(currMod)
            }
        }
    }
    //displayMsg
} // findRefs

// Initializations


void restoreRefs(string projectName)
{
	currProj = project(projectName) //current
	firstMsg = true

	// Main

	if (null(currProj)) {
		warningBox("This tool must be run within a project.")
	} else {
		findRefs
	}
}